"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useApplePay = void 0;
var _react = require("react");
var _reactNative = require("react-native");
var _reactNativeWalletPayments = require("react-native-wallet-payments");
const {
  WalletPaymentsApplePayEventEmitter
} = _reactNative.NativeModules;
const applePayEmitter = new _reactNative.NativeEventEmitter(WalletPaymentsApplePayEventEmitter);
const useApplePay = (onShippingContactSelected, onShippingMethodSelected) => {
  const subscriptions = (0, _react.useRef)([]);
  (0, _react.useEffect)(() => {
    const contactSubscription = applePayEmitter.addListener('onShippingContactSelected', contact => {
      onShippingContactSelected?.(contact);
    });
    const methodSubscription = applePayEmitter.addListener('onShippingMethodSelected', method => {
      onShippingMethodSelected?.(method);
    });
    subscriptions.current = [contactSubscription, methodSubscription];
    return () => {
      subscriptions.current.forEach(sub => sub.remove());
    };
  }, [onShippingContactSelected, onShippingMethodSelected]);

  /**
   * startPayment
   * This function starts the Apple Pay payment process with the provided payment request options.
   * @param {PaymentRequest} options - The payment request details including merchant info, items, and optional fields.
   * @returns {Promise<PaymentResult>} - A promise that resolves with the payment result.
   * @throws {Error} - An error is thrown if the payment process fails.
   * @example
   * const result = await startPayment({
   *    merchantIdentifier: 'merchant.com.example',
   *    countryCode: 'US',
   *    currencyCode: 'USD',
   *    supportedNetworks: ['visa', 'masterCard', 'amex'],
   *    merchantCapabilities: ['3DS'],
   *    items: [
   *        { label: 'Product 1', amount: '10.00' },
   *        { label: 'Product 2', amount: '5.00' },
   *    ],
   * });
   */
  const startPayment = async options => {
    return new Promise((resolve, reject) => {
      const paymentAuthorizedSubscription = applePayEmitter.addListener('onPaymentAuthorized', paymentInfo => {
        paymentAuthorizedSubscription.remove();
        resolve(paymentInfo);
      });
      (0, _reactNativeWalletPayments.showPaymentSheet)(options).catch(error => {
        paymentAuthorizedSubscription.remove();
        reject(error);
      });
    });
  };
  const confirmPayment = (0, _react.useCallback)(async () => {
    await (0, _reactNativeWalletPayments.confirmPayment)();
  }, []);
  const rejectPayment = (0, _react.useCallback)(async () => {
    await (0, _reactNativeWalletPayments.rejectPayment)();
  }, []);
  return {
    startPayment,
    confirmPayment,
    rejectPayment
  };
};
exports.useApplePay = useApplePay;
//# sourceMappingURL=useApplePay.js.map